<?php
/*
 * Copyright (c) 2022. IOWEB TECHNOLOGIES
 */

namespace Ioweb\Base\Api;

/**
 * interface for creating an attribute setup data
 * Check \Magento\Catalog\Model\ResourceModel\Setup\PropertyMapper::map for available properties
 */
interface AttributeSetupDataInterface
{
    const KEY_APPLY_TO = 'apply_to';
    const KEY_BACKEND = 'backend';
    const KEY_CLASS = 'class';
    const KEY_COMPARABLE = 'comparable';
    const KEY_DEFAULT = 'default';
    const KEY_FILTERABLE = 'filterable';
    const KEY_FRONTEND = 'frontend';
    const KEY_GLOBAL = 'global';
    const KEY_INPUT = 'input';
    const KEY_LABEL = 'label';
    const KEY_REQUIRED = 'required';
    const KEY_SEARCHABLE = 'searchable';
    const KEY_SOURCE = 'source';
    const KEY_TYPE = 'type';
    const KEY_UNIQUE = 'unique';
    const KEY_USED_IN_PRODUCT_LISTING = 'used_in_product_listing';
    const KEY_USER_DEFINED = 'user_defined';
    const KEY_VISIBLE = 'visible';
    const KEY_IS_VISIBLE = 'is_visible';
    const KEY_VISIBLE_IN_FRONT = 'visible_on_front';
    const KEY_IS_USED_IN_GRID = 'is_used_in_grid';
    const TYPE_TEXT = 'text';
    const TYPE_INT = 'int';
    const TYPE_VARCHAR = 'varchar';
    const INPUT_BOOLEAN = 'boolean';
    const INPUT_TEXT = 'text';


    /**
     * @return bool
     */
    public function getIsUsedInGrid();

    /**
     * @param bool $isUsedInGrid
     * @return $this
     */
    public function setIsUsedInGrid($isUsedInGrid);

    /**
     * @param string $applyTo
     * @return $this
     */
    public function setApplyTo($applyTo);

    /**
     * @return string
     */
    public function getApplyTo();

    /**
     * @param string $backend
     * @return $this
     */
    public function setBackend($backend);

    /**
     * @return string
     */
    public function getBackend();

    /**
     * @param string $class
     * @return $this
     */
    public function setClass($class);

    /**
     * @return string
     */
    public function getClass();

    /**
     * @param bool $comparable
     * @return $this
     */
    public function setComparable($comparable);

    /**
     * @return bool
     */
    public function getComparable();

    /**
     * @param string $default
     * @return $this
     */
    public function setDefault($default);

    /**
     * @return string
     */
    public function getDefault();

    /**
     * @param bool $filterable
     * @return $this
     */
    public function setFilterable($filterable);

    /**
     * @return bool
     */
    public function getFilterable();

    /**
     * @param string $frontend
     * @return $this
     */
    public function setFrontend($frontend);

    /**
     * @return string
     */
    public function getFrontend();

    /**
     * @param int $global
     * @return $this
     */
    public function setGlobal($global);

    /**
     * @return int
     */
    public function getGlobal();

    /**
     * @param string $input
     * @return $this
     */
    public function setInput($input);

    /**
     * @return string
     */
    public function getInput();

    /**
     * @param string $label
     * @return $this
     */
    public function setLabel($label);

    /**
     * @return string
     */
    public function getLabel();

    /**
     * @param bool $required
     * @return $this
     */
    public function setRequired($required);

    /**
     * @return bool
     */
    public function getRequired();

    /**
     * @param bool $searchable
     * @return $this
     */
    public function setSearchable($searchable);

    /**
     * @return bool
     */
    public function getSearchable();

    /**
     * @param string $source
     * @return $this
     */
    public function setSource($source);

    /**
     * @return string
     */
    public function getSource();

    /**
     * @param string $type
     * @return $this
     */
    public function setType($type);

    /**
     * @return string
     */
    public function getType();

    /**
     * @param bool $unique
     * @return $this
     */
    public function setUnique($unique);

    /**
     * @return bool
     */
    public function getUnique();

    /**
     * @param bool $usedInProductListing
     * @return $this
     */
    public function setUsedInProductListing($usedInProductListing);

    /**
     * @return bool
     */
    public function getUsedInProductListing();

    /**
     * @param bool $userDefined
     * @return $this
     */
    public function setUserDefined($userDefined);

    /**
     * @return bool
     */
    public function getUserDefined();

    /**
     * @param bool $visible
     * @return $this
     */
    public function setVisible($visible);

    /**
     * @return bool
     */
    public function getVisible();

    /**
     * @param bool $visibleInFront
     * @return $this
     */
    public function setVisibleInFront($visibleInFront);

    /**
     * @return bool
     */
    public function getVisibleInFront();

}