<?php
    /**
     * Copyright (c) 2019. IOWEB TECHNOLOGIES
     */

    namespace Ioweb\Base\Helper;


    use Ioweb\Base\Model\AttributeSetupData;
    use Magento\Catalog\Model\Product;
    use Magento\Eav\Model\AttributeRepository;
    use Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface;
    use Magento\Framework\App\Helper\AbstractHelper;
    use Magento\Framework\App\Helper\Context;
    use Magento\Framework\Exception\NoSuchEntityException;
    use Magento\Framework\Setup\ModuleDataSetupInterface;

    class Attribute extends AbstractHelper
    {
        private $eavSetupFactory;
        private $attributeRepository;
        private $moduleDataSetup;
        protected $config;
        protected $eavSetup;
        protected $attributeManagement;


        public function __construct(
            Context $context,
            \Magento\Eav\Setup\EavSetupFactory $eavSetupFactory,
            AttributeRepository $attributeRepository,
            ModuleDataSetupInterface $moduleDataSetup,
            \Magento\Eav\Api\AttributeManagementInterface $attributeManagement,
            \Magento\Eav\Setup\EavSetup                   $eavSetup,
            \Magento\Catalog\Model\Config                 $config
        )
        {
            parent::__construct($context);
            $this->eavSetupFactory = $eavSetupFactory;
            $this->attributeRepository = $attributeRepository;
            $this->moduleDataSetup = $moduleDataSetup;
            $this->attributeManagement = $attributeManagement;
            $this->eavSetup = $eavSetup;
            $this->config = $config;
        }

        /**
         * @param $code
         * for text input=text, type=text
         * for yes/no input=boolean, type=int
         */
        public function createProductAttribute(
            $code,
            $type = 'text',
            $input = 'text',
            $visible = true,
            $required = false,
            $userDefined = true,
            $default = '',
            $searchable = false,
            $filterable = false,
            $comparable = false,
            $visibleInFront = false,
            $unique = false,
            $source = '',
            $global = ScopedAttributeInterface::SCOPE_GLOBAL
        )
        {


            try {
                $attribute = $this->attributeRepository->get(Product::ENTITY, $code);


            } catch (NoSuchEntityException $noSuchEntityException) {
                $this->moduleDataSetup->startSetup();
                /**
                 * @var $eavSetup \Magento\Eav\Setup\EavSetup
                 */
                $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);
                $eavSetup->addAttribute(
                    Product::ENTITY,
                    $code,
                    [
                        'type' => $type,
                        'backend' => '',
                        'frontend' => '',
                        'label' => $code,
                        'input' => $input,
                        'class' => '',
                        'source' => $source,
                        'global' => $global,
                        'visible' => $visible,
                        'required' => $required,
                        'user_defined' => $userDefined,
                        'default' => $default,
                        'searchable' => $searchable,
                        'filterable' => $filterable,
                        'comparable' => $comparable,
                        'visible_on_front' => $visibleInFront,
                        'used_in_product_listing' => true,
                        'unique' => $unique,
                        'apply_to' => '',
                    ]
                );
                $this->moduleDataSetup->endSetup();
            }
        }


        /**
         * @param $code
         * for text input=text, type=text
         * for yes/no input=boolean, type=int
         */
        public function createCustomAttribute(
            $code,
            AttributeSetupData $attributeSetupData
        )
        {
            try {
                $attribute = $this->attributeRepository->get(Product::ENTITY, $code);

            } catch (NoSuchEntityException $noSuchEntityException) {
                $this->moduleDataSetup->startSetup();
                /**
                 * @var $eavSetup \Magento\Eav\Setup\EavSetup
                 */
                $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);
                $eavSetup->addAttribute(
                    Product::ENTITY,
                    $code,
                    $attributeSetupData->getData()
                );
                $this->moduleDataSetup->endSetup();
            }
        }

        public function assignToAllAttributeSets($attributeCode, $group){
            $attribute = $this->attributeRepository->get(Product::ENTITY, $attributeCode);
            $entityTypeId = $this->eavSetup->getEntityTypeId(\Magento\Catalog\Model\Product::ENTITY);
            $attributeSetIds = $this->eavSetup->getAllAttributeSetIds($entityTypeId);

            foreach ($attributeSetIds as $attributeSetId) {
                if ($attributeSetId) {
                    $group_id = $this->config->getAttributeGroupId($attributeSetId, $group);
                    $this->attributeManagement->assign(
                        Product::ENTITY,
                        $attributeSetId,
                        $group_id,
                        $attributeCode,
                        2
                    );
                }
            }
        }

    }