<?php
/*
 * Copyright (c) 2020. IOWEB TECHNOLOGIES
 */

namespace Ioweb\Base\Helper;


use Symfony\Component\Console\Formatter\OutputFormatter;
use Psr\Log\LoggerInterface;
use Symfony\Component\Console\Helper\ProgressBar;
use Symfony\Component\Console\Input\ArgvInput;
use Symfony\Component\Console\Output\ConsoleOutput;
use Symfony\Component\Console\Output\ConsoleOutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class CliLogger implements LoggerInterface
{


    protected $errOutput;
    protected $logger;
    protected $output;
    protected $dir;

    public function __construct(
        \Magento\Framework\App\Filesystem\DirectoryList $dir,
        \Ioweb\Base\Model\Logger $logger
    )
    {
        $input = new ArgvInput();
        $output = new ConsoleOutput();
        $errOutput = $output instanceof ConsoleOutputInterface ? $output->getErrorOutput() : $output;
        $this->logger = $logger;
        $this->output = new SymfonyStyle($input, $output);
        $this->errOutput = new SymfonyStyle($input, $errOutput);
        $this->dir = $dir;
    }

    public function alert($message, array $context = array())
    {
        $this->errOutput->error($message);
        $this->logger->alert($message, $context);
    }

    public function critical($message, array $context = array())
    {
        $this->errOutput->error($message);
        $this->logger->critical($message, $context);
    }

    public function debug($message, array $context = array())
    {
        $this->output->writeln($message);
        $this->logger->debug($message, $context);
    }

    public function emergency($message, array $context = array())
    {
        $this->errOutput->error($message);
        $this->logger->emergency($message, $context);
    }

    public function error($message, array $context = array())
    {
        $this->errOutput->error($message);
        $this->logger->error($message, $context);
    }

    public function err($message, array $context = array())
    {
        $this->errOutput->error($message);
        $this->logger->error($message, $context);
    }

    public function getProgressBar($count)
    {
        ProgressBar::setFormatDefinition('custom', "%current%/%max% %message%");
        $progressBar = new ProgressBar($this->output, $count);
        $progressBar->setFormat('custom');
        return $progressBar;
    }

    public function info($message, array $context = array())
    {
        $this->output->writeln($message);
        $this->logger->notice($message, $context);
    }

    public function log($level, $message, array $context = array())
    {
        $this->output->writeln($message);
        $this->logger->log($level, $message, $context);
    }

    public function notice($message, array $context = array())
    {
        $this->output->writeln($message);
        $this->logger->notice($message, $context);
    }

    public function setCustomFilename($filename)
    {
        $this->logger->pushHandler(new \Monolog\Handler\StreamHandler($this->dir->getRoot().'/var/log/' . $filename));
    }

    public function warning($message, array $context = array())
    {
        $this->errOutput->warning($message);
        $this->logger->warning($message, $context);
    }

    public function warn($message, array $context = array())
    {
        $this->errOutput->warning($message);
        $this->logger->warning($message, $context);
    }

    public function success($message, array $context = array()){
        $this->output->success($message);
        $this->logger->info($message);
    }

}
