<?php
/*
 * Copyright (c) 2022. IOWEB TECHNOLOGIES
 */

namespace Ioweb\Base\Helper;


use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Magento\Framework\Stdlib\CookieManagerInterface;

class Cookie
{
    protected $cookieManager;
    protected $cookieMetadataFactory;
    protected $scopeConfig;

    public function __construct(CookieManagerInterface $cookieManager,
                                CookieMetadataFactory  $cookieMetadataFactory,
                                ScopeConfigInterface   $scopeConfig
    )
    {
        $this->cookieManager = $cookieManager;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * get cookie by name
     */
    public function getCookie($cookieName)
    {
        return $this->cookieManager->getCookie($cookieName);

    }

    /**
     * set public cookie
     */
    public function setPublicCookie($cookieName, $value)
    {

        $metadata = $this->cookieMetadataFactory->createPublicCookieMetadata()->setDuration(
            86400
        ) // Cookie will expire after one day (86400 seconds)
        ->setSecure(true) //the cookie is only available under HTTPS
        ->setHttpOnly(false); // cookies can be accessed by JS

        $this->cookieManager->setPublicCookie(
            $cookieName,
            $value,
            $metadata
        );
    }

    /**
     * set sensitive cookie
     */
    public function setSensitiveCookie($cookieName, $value)
    {

        $metadata = $this->cookieMetadataFactory->createSensitiveCookieMetadata()->setPath('')// for global
//        ->setDomain(
//            '.example.com'
//        )
        ;// cookie will be available for www.example.com, example.com and will not be available for anotherexample.com

        $this->cookieManager->setSensitiveCookie(
            $cookieName,
            $value,
            $metadata
        );
    }
}