<?php
/**
 * Copyright (c) 2019. IOWEB TECHNOLOGIES
 */

namespace Ioweb\Base\Helper;


use Ioweb\Base\Model\AttributeSetupData;
use Magento\Catalog\Model\Product;
use Magento\Customer\Model\Customer;
use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Eav\Api\Data\AttributeSetInterfaceFactory;
use Magento\Eav\Model\AttributeRepository;
use Magento\Eav\Model\Config;
use Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Setup\ModuleDataSetupInterface;

class CustomerAttribute extends AbstractHelper
{
    protected $attributeManagement;
    protected $config;
    protected $eavSetup;
    private $attributeRepository;
    private $eavSetupFactory;
    private $moduleDataSetup;
    private $customerSetupFactory;
    private $attributeSetFactory;

    public function __construct(
        Context                                       $context,
        \Magento\Eav\Setup\EavSetupFactory            $eavSetupFactory,
        AttributeRepository                           $attributeRepository,
        ModuleDataSetupInterface                      $moduleDataSetup,
        \Magento\Eav\Api\AttributeManagementInterface $attributeManagement,
        \Magento\Eav\Setup\EavSetup                   $eavSetup,
        \Magento\Catalog\Model\Config                 $config,
        Config                                        $eavConfig,
        CustomerSetupFactory $customerSetupFactory,
        AttributeSetInterfaceFactory $attributeSetFactory
    )
    {
        parent::__construct($context);
        $this->eavSetupFactory = $eavSetupFactory;
        $this->attributeRepository = $attributeRepository;
        $this->moduleDataSetup = $moduleDataSetup;
        $this->attributeManagement = $attributeManagement;
        $this->eavSetup = $eavSetup;
        $this->config = $config;
        $this->eavConfig = $eavConfig;
        $this->customerSetupFactory = $customerSetupFactory;
        $this->attributeSetFactory = $attributeSetFactory;
    }

    public function addAttributeToForms($attributeCode, $forms = ['adminhtml_customer'])
    {
        $this->moduleDataSetup->startSetup();
        $customerSetup = $this->customerSetupFactory->create(['setup' => $this->moduleDataSetup]);

        $customerEntity = $customerSetup->getEavConfig()->getEntityType('customer');
        $attributeSetId = $customerEntity->getDefaultAttributeSetId();

        $attributeSet = $this->attributeSetFactory->create();
        $attributeGroupId = $attributeSet->getDefaultGroupId($attributeSetId);


        $attribute = $customerSetup->getEavConfig()->getAttribute(Customer::ENTITY, $attributeCode)
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => $forms,//you can use other forms also
            ]);

        $attribute->save();
        $this->moduleDataSetup->endSetup();
    }

    /**
     * @param $code
     * for text input=text, type=text
     * for yes/no input=boolean, type=int
     */
    public function createCustomAttribute(
        $code,
        AttributeSetupData $attributeSetupData
    )
    {
        try {
            $attribute = $this->attributeRepository->get(Customer::ENTITY, $code);

        } catch (NoSuchEntityException $noSuchEntityException) {
            $this->moduleDataSetup->startSetup();
            /**
             * @var $eavSetup \Magento\Eav\Setup\EavSetup
             */
            $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);
            $eavSetup->addAttribute(
                Customer::ENTITY,
                $code,
                $attributeSetupData->getData()
            );
            $this->moduleDataSetup->endSetup();
        }
    }

    /**
     * @return Config
     */
    public function getEavConfig(): Config
    {
        return $this->eavConfig;
    }

}