<?php
/**
 * Copyright (c) 2019. IOWEB TECHNOLOGIES
 */

/**
 * Created by IntelliJ IDEA.
 * User: gabtz
 * Date: 4/22/2019
 * Time: 3:12 PM
 */

namespace Ioweb\Base\Helper;


use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\ProductRepository;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory as CategoryCollectionFactory;
use Magento\Catalog\Model\ResourceModel\Product\Action;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\CatalogInventory\Api\Data\StockItemInterface;
use Magento\CatalogInventory\Api\StockItemCriteriaInterfaceFactory;
use Magento\CatalogInventory\Model\Stock\Status;
use Magento\CatalogInventory\Model\Stock\StockItemRepository;
use Magento\CatalogInventory\Model\StockRegistry;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\StoreManagerInterface;

class ProductHelper extends AbstractHelper
{


    protected $categoryCollectionFactory;
    protected $categoryRepository;
    protected $collectionFactory;
    protected $mediaUrl;
    protected $productModel;
    protected $productRepository;
    protected $resourceAction;
    protected $searchCriteriaBuilderFactory;
    protected $stockItemCriteriaInterfaceFactory;
    protected $stockRegistry;
    protected $stockRepository;
    protected $storeManager;

    public function __construct(Context                                      $context,
                                ProductRepository                            $productRepository,
                                SearchCriteriaBuilderFactory                 $searchCriteriaBuilderFactory,
                                \Magento\Catalog\Model\ResourceModel\Product $productModel,
                                Action                                       $resourceAction,
                                StoreManagerInterface                        $storeManager,
                                CollectionFactory                            $collectionFactory,
                                StockItemRepository                          $stockRepository,
                                StockItemCriteriaInterfaceFactory            $stockItemCriteriaInterfaceFactory,
                                CategoryCollectionFactory                    $categoryCollectionFactory,
                                CategoryRepositoryInterface                  $categoryRepository,
                                StockRegistry                                $stockRegistry
    )
    {
        $this->resourceAction = $resourceAction;
        $this->productRepository = $productRepository;
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
        $this->productModel = $productModel;
        $this->collectionFactory = $collectionFactory;
        $this->storeManager = $storeManager;
        $this->stockRepository = $stockRepository;
        $this->stockItemCriteriaInterfaceFactory = $stockItemCriteriaInterfaceFactory;
        $this->stockRegistry = $stockRegistry;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
        $this->categoryRepository = $categoryRepository;
        $this->mediaUrl = null;
        parent::__construct($context);
    }

    /**
     * @param $product ProductInterface|Product
     */
    public function getAdditionalImages($product)
    {
        if (empty($this->mediaUrl)) {
            $this->mediaUrl = $this->storeManager->getStore()
                ->getBaseUrl(
                    UrlInterface::URL_TYPE_MEDIA
                );
        }
        $additional = [];
        $mainImage = $product->getImage();
        $gallery = $product->getMediaGalleryEntries();
        foreach ($gallery as $entry) {
            if (strpos($entry->getFile(), $mainImage) === false) {
                $additional[] = $this->mediaUrl . 'catalog/product' . $entry->getFile();
            }
        }
        return $additional;
    }

    /**
     * @param $categoryId
     * @return mixed
     */
    public function getCategory($categoryId, $storeId = null)
    {
        return $this->categoryRepository->get($categoryId, $storeId);
    }

    public function getCategoryId($product)
    {
        $categoriesCollection = $product->getCategoryCollection();

        $categories = $categoriesCollection
            ->addAttributeToSelect(array('name', 'level'))
            ->addAttributeToFilter('is_active', 1)//only active categories
            ->addAttributeToSort('level', 'DESC');

        $categoryId = '';
        foreach ($categories as $_category) {
            $categoryId = $_category->getId();
            break;
        }
        return $categoryId;
    }

    /**
     * @param $product ProductInterface|Product
     */
    public function getCategoryPath($product)
    {
        $categoriesCollection = $product->getCategoryCollection();

        $categories = $categoriesCollection
            ->addAttributeToSelect(array('name', 'level'))
            ->addAttributeToFilter('is_active', 1)//only active categories
            ->addAttributeToSort('level', 'DESC');

        $categoryPath = '';
        $categoryId = '';
        $category = null;
        foreach ($categories as $_category) {
            $category = $_category;
            $categoryId = $_category->getId();
            break;
        }

        if (empty($category)) {
            return null;
        }
        $parents = $this->getCategory($category->getId())->getParentCategories();
        $parentIds = array();

        foreach ($parents as $parentCategory) {
            array_push($parentIds, $parentCategory->getId());
        }

        $parentsCollection = $this->categoryCollectionFactory->create()
            ->addAttributeToFilter('entity_id', array('in' => $parentIds))
            ->addAttributeToFilter('is_active', 1)//only active categories
            ->addAttributeToSort('level', 'ASC')
            ->addAttributeToSelect(array('name', 'level'));

        foreach ($parentsCollection as $parentCategory) {
            if (!empty($categoryPath)) {
                $categoryPath .= ' - ';
            }
            $categoryPath .= $parentCategory->load($parentCategory->getId())
                ->getName();
            $parentCategory->clearInstance();
        }
        return $categoryPath;
    }

    /**
     * @return []
     */
    public function getEnabledProducts()
    {

        $collection = $this->collectionFactory->create();
        $collection->addAttributeToSelect(['id', 'status'])
            ->addAttributeToFilter('type_id', ['neq' => Configurable::TYPE_CODE])
            ->addAttributeToFilter('status', ['eq' => Product\Attribute\Source\Status::STATUS_ENABLED]);
        return $collection->getAllIds();
    }

    public function getProductById($id)
    {
        return $this->productRepository->getById($id);
    }

    /**
     * @return StockItemInterface[]
     */
    public function getStockState()
    {
        $criteria = $this->stockItemCriteriaInterfaceFactory->create();
        $stockData = $this->stockRepository->getList($criteria);
        return $stockData->getItems();
    }

    /**
     * Saves only a single product attribute value for performance.
     * @param $product
     * @param $attributeCode
     * @param $value
     * @throws
     */
    public function saveAttribute($product, $attributeCode, $value)
    {
        $productResource = $this->productModel;
        $product->setData($attributeCode, $value);
        $productResource->saveAttribute($product, $attributeCode);
        unset($product);
    }

    /**
     * @param $product ProductInterface
     * @param $qty
     * @param $inStock
     * @throws
     */
    public function saveQty($product, $qty)
    {
        $stockItem = $this->stockRepository->get($product->getData('item_id'));
        $stockItem->setQty($qty);
        $stockItem->setIsInStock(Status::STATUS_IN_STOCK);
        $this->stockRepository->save($stockItem);
        unset($stockItem);
    }

    /**
     * @param $product
     * @param $qty
     * @throws NoSuchEntityException
     */
    public function saveQtyUsingRegistry($product, $qty)
    {
        $sku = $product->getSku();
        $stockItem = $this->stockRegistry->getStockItemBySku($sku);
        $stockItem->setQty($qty);
        $this->stockRegistry->updateStockItemBySku($sku, $stockItem);
    }

    /**
     * @param $ids
     * @param $attributeCode
     * @param $value
     * @throws NoSuchEntityException
     */
    public function setAttributeInProducts($ids, $attributeCode, $value)
    {
        $updateAttributes[$attributeCode] = $value;
        $this->resourceAction->updateAttributes($ids, $updateAttributes, $this->storeManager->getStore()->getId());
    }


}