<?php
/*
 * Copyright (c) 2021. IOWEB TECHNOLOGIES
 */

namespace Ioweb\Base\Helper;


use Magento\Catalog\Model\Product\Gallery\ReadHandler;
use Magento\CatalogInventory\Model\Stock\StockItemRepository;
use Magento\ConfigurableProduct\Api\LinkManagementInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\StoreManagerInterface;

class Setup extends AbstractHelper
{

    const IOWEB_SERVER = 'https://licensing.ioweb.gr';

    const IOWEB_INSTALLATION_REGISTRATION_PATH = "/installation/register";

    /**
     * @var StoreManagerInterface
     */
    private $_storeManager;

    private $_urlInterface;

    private $_galleryReadHandler;

    private $stockItemRepository;

    private $configWriter;

    private $linkManagement;

    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        UrlInterface $urlInterface,
        ReadHandler $galleryReadHandler,
        StockItemRepository $stockItemRepository,
        WriterInterface $configWriter,
        LinkManagementInterface $linkManagement
    )
    {
        $this->_urlInterface = $urlInterface;
        $this->_storeManager = $storeManager;
        $this->_galleryReadHandler = $galleryReadHandler;
        $this->stockItemRepository = $stockItemRepository;
        $this->configWriter = $configWriter;
        $this->linkManagement = $linkManagement;
        parent::__construct($context);
    }


    public function getInstallationId($moduleName, $moduleVersion, $configPath)
    {
        try {
            $url = self::IOWEB_SERVER . self::IOWEB_INSTALLATION_REGISTRATION_PATH;
            $client = new \Zend_Http_Client($url);
            $client->setMethod(\Zend_Http_Client::POST);
            //    $client->setHeaders('Content-type', 'application/x-www-form-urlencoded');
            $client
                ->setHeaders(
                    \Zend_Http_Client::CONTENT_TYPE,
                    \Zend_Http_Client::ENC_URLENCODED
                );

            $client->setParameterPost('application', $moduleName);
            $client->setParameterPost('version', $moduleVersion);
            $client->setParameterPost(
                'domain',
                $this->_storeManager->getStore()
                    ->getBaseUrl()
            );
            $response = $client->request();

            if ($response->isSuccessful()) {
                $json = json_decode($response->getBody());
                $info = json_decode($json);;
                $this->saveConfig(
                    $configPath . '/support/installation_id',
                    $info->installation
                );
            }
        } catch (\Exception $exception) {
            $this->_logger->critical($exception->getMessage());
        }
    }

    public function saveConfig(
        $path,
        $value,
        $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT
    )
    {
        $this->configWriter->save($path, $value, $scope);

        return $this;
    }
}