<?php
/**
 * Copyright (c) 2020. IOWEB TECHNOLOGIES
 */

namespace Ioweb\Base\Helper;


use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product;
use Magento\CatalogInventory\Api\StockStateInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\ObjectManagerInterface;
use Magento\InventoryCatalogApi\Api\DefaultStockProviderInterface;
use Magento\InventorySalesApi\Api\AreProductsSalableInterface;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\InventorySalesApi\Api\StockResolverInterface;
use Magento\Store\Api\WebsiteRepositoryInterface;
use Magento\Store\Model\StoreManagerInterface;

class Stock extends AbstractHelper
{

    protected $stockState;
    private $appHelper;
    /**
     * @var AreProductsSalableInterface
     */
    private $areProductsSalable;
    /**
     * @var DefaultStockProviderInterface
     */
    private $defaultStockProvider;
    /**
     * @var StockResolverInterface
     */
    private $stockResolver;
    private $storeManager;
    /**
     * @var WebsiteRepositoryInterface
     */
    private $websiteRepository;

    public function __construct(
        Context                       $context,
        AreProductsSalableInterface   $areProductsSalable,
        StockResolverInterface        $stockResolver,
        WebsiteRepositoryInterface    $websiteRepository,
        DefaultStockProviderInterface $defaultStockProvider,
        App                           $appHelper,
        StockStateInterface           $stockState,
        StoreManagerInterface         $storeManager
    )
    {
        parent::__construct($context);
        $this->areProductsSalable = $areProductsSalable;
        $this->stockResolver = $stockResolver;
        $this->websiteRepository = $websiteRepository;
        $this->defaultStockProvider = $defaultStockProvider;
        $this->appHelper = $appHelper;
        $this->stockState = $stockState;
        $this->storeManager = $storeManager;
    }


    /**
     * @param $product ProductInterface|Product
     */
    public function getHasStock($product)
    {

        if (version_compare($this->appHelper->getVersion(), '2.4.0', "<")) {
            return $this->stockState->verifyStock($product->getId());
        } else {
            $websiteCode = $this->storeManager->getWebsite()->getCode();
            $stock = $this->stockResolver->execute(SalesChannelInterface::TYPE_WEBSITE, $websiteCode);
            return $product->isSaleable();
        }
    }

    public function getQty($product)
    {
        return (float)$this->stockState->getStockQty(
            $product->getId(),
            $this->storeManager->getStore()->getWebsiteId()
        );
    }

}